package mixer;

import java.util.ArrayList;
import java.util.List;

class Graph
{
	private int n;
    private int nDiv2;
	private Node[] nodes;
	protected Graph(int[] permutation){
		n = permutation.length; // n = 2^k
        nDiv2 = n >> 1;
        createNodes();
        createEdges(permutation);
        setSwitches();
	}

    // provide an access to graph to algorithm result
    // index must be less then n/2
	protected boolean getSwitchValue(int index,boolean up)
    {
        return up ? nodes[index].on : nodes[index + n / 2].on;
    }

    // create two lines of nodes size n/2 each
    // the value of the i th node is (i,i+n/2) if i < n /2  (first line)
    // otherwise its value is (i - n/2 , i)                 (second line)
	private void createNodes()
    {
        nodes = new Node[n];
        for (int i = 0; i < n / 2; i++)
        {
            nodes[i] = new Node(true);
            nodes[i + nDiv2] = new Node(false);
        }
    }

    // create an edge between each pair of nodes i,j from different lines (i index of the first line)
    // if exists k in  i th node's value and t in j th node's value
    // s.t permutation[k] == t
    // the edge is broken if (k < n/2 and t >= n/2) or (k >= n/2 and t < n/2)
    // Note: in purpose to avoid edge cases, each node has exactly two edges
	private void createEdges(int[] permutation)
    {
        int j;
        for (int i = 0; i < nDiv2; i++)
        {
            j = (permutation[i] % nDiv2) + nDiv2;
            nodes[i].edges.add(new Edge(nodes[j], (permutation[i] >= nDiv2)));
            nodes[j].edges.add(new Edge(nodes[i], (permutation[i] >= nDiv2)));

            j = (permutation[i + nDiv2] % nDiv2) + nDiv2;
            nodes[i].edges.add(new Edge(nodes[j], (permutation[i + nDiv2] < nDiv2)));
            nodes[j].edges.add(new Edge(nodes[i], (permutation[i + nDiv2] < nDiv2)));
        }
    }

    // set switch's value (on/off) for each switch (node)
    // s.t if nodes i,j connected by edge e, i th switch's value
    // must be equal to j's if e is broken or not equal if e is not broken
	private void setSwitches()
    {
        Node node;
        boolean v;
        Edge e0,e1;
        // iterate over first line of nodes
        for (int i = 0; i < nDiv2; i++)
        {
            node = nodes[i];
            if (node.set)
                continue;
            //select default value for first node in connected component
            v = false;
            // set value to all reachable nodes from node
            while (true)
            {
                node.set = true;
                node.on = v;
                e0 = node.edges.get(0); e1 = node.edges.get(1);
                if (e0.neighbor.set && e1.neighbor.set)
                    break;
                v ^= (!e0.neighbor.set) ? e0.broken : e1.broken;
                node = (!e0.neighbor.set) ? e0.neighbor : e1.neighbor;
            }
        }
    }

    //inner classes

	private class Node
	{
		public boolean up;
		public List<Edge> edges;
		public boolean on;
		public boolean set;
		public Node(boolean up)
		{
			this.up = up;
			edges = new ArrayList<Edge>();
			set = false;
		} 
	}

	private class Edge
	{
		public Node neighbor;
		public boolean broken;
		public Edge(Node neighbor, boolean broken)
		{
			this.neighbor = neighbor;
			this.broken = broken;
		}
	}

	
}

