package verifier;

import meerkat.crypto.Encryption;
import meerkat.crypto.mixnet.Mix2ZeroKnowledgeVerifier;
import meerkat.protobuf.ConcreteCrypto.ElGamalCiphertext;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;
import necessary.General;
import necessary.Group;


public class Verifier implements Mix2ZeroKnowledgeVerifier {

    /**
     * Return true iff the proof is valid.
     * @param in1
     * @param in2
     * @param out1
     * @param out2
     * @return
     */

    Group group;
    General general;

    public Verifier(Group group,General general) {
        this.group = group;
        this.general = general;
    }

    public boolean verify(Crypto.RerandomizableEncryptedMessage in1,
                          Crypto.RerandomizableEncryptedMessage in2,
                          Crypto.RerandomizableEncryptedMessage out1,
                          Crypto.RerandomizableEncryptedMessage out2,
                          Mixing.ZeroKnowledgeProof proof)
    {

        ElGamalCiphertext e1ElGamal = general.calcRerandomizable2ElGamal(in1);
        ElGamalCiphertext e2ElGamal = general.calcRerandomizable2ElGamal(in2);
        ElGamalCiphertext e1TagElGamal = general.calcRerandomizable2ElGamal(out1);
        ElGamalCiphertext e2TagElGamal = general.calcRerandomizable2ElGamal(out2);

        return verifyElGamaOrProof(e1ElGamal,e2ElGamal,e1TagElGamal,e2TagElGamal,proof.getFirst()) &&
               verifyElGamaOrProof(e1ElGamal,e2ElGamal,e1TagElGamal,e2TagElGamal,proof.getSecond()) &&
               verifyElGamaOrProof(e1ElGamal,e2ElGamal,e1TagElGamal,e2TagElGamal,proof.getThird()) &&
               verifyElGamaOrProof(e1ElGamal,e2ElGamal,e1TagElGamal,e2TagElGamal,proof.getFourth());
    }

    public boolean verifyElGamaOrProof(ElGamalCiphertext e1,
                                 ElGamalCiphertext e2,
                                 ElGamalCiphertext e1New,
                                 ElGamalCiphertext e2New,
                                 Mixing.ZeroKnowledgeProof.OrProof orProof)
    {

        return   //input
                 orProof.getG1().equals(group.getG())&&
                 orProof.getH1().equals(group.div(e1New.getC1(), e1.getC1()))&&
                 orProof.getG2().equals(group.getH())&&
                 orProof.getH2().equals(group.div(e1New.getC2(), e1.getC2()))&&
                 // input'
                 orProof.getG1Tag().equals(group.getG())&&
                 orProof.getH1Tag().equals(group.div(e2New.getC1(), e2.getC1()))&&
                 orProof.getG2Tag().equals(group.getH())&&
                 orProof.getH2Tag().equals(group.div(e2New.getC2(), e2.getC2())) &&
                 // hash
                 // assert (c1 + c2 ) % group size == hash (imput + step1) % group size
                 general.mod((general.add(orProof.getC1(),orProof.getC2())),group.groupSize())
                 .equals(general.mod(general.hash(orProof.getG1(), orProof.getH1(), orProof.getG2(), orProof.getH2(),
                         orProof.getG1Tag(), orProof.getH1Tag(), orProof.getG2Tag(), orProof.getH2Tag(),
                         orProof.getV(),orProof.getU(),orProof.getVTag(),orProof.getUTag()) , group.groupSize()))&&
                 // proof
                 // g1 ^ z == u * ( h1 ^ c1 ) && g2 ^ z == v * ( h2 ^ c1 ) && the same for tag case
                 group.pow(orProof.getG1(), orProof.getZ()).equals(group.mul(orProof.getU(), group.pow(orProof.getH1(),orProof.getC1()))) &&
                 group.pow(orProof.getG2(), orProof.getZ()).equals(group.mul(orProof.getV(), group.pow(orProof.getH2(),orProof.getC1()))) &&
                 group.pow(orProof.getG1Tag(), orProof.getZTag()).equals(group.mul(orProof.getUTag(), group.pow(orProof.getH1Tag(),orProof.getC2()))) &&
                 group.pow(orProof.getG2Tag(), orProof.getZTag()).equals(group.mul(orProof.getVTag(), group.pow(orProof.getH2Tag(),orProof.getC2())));

    }
}
